/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project.visual;

import java.io.IOException;
import java.util.ArrayList;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import nl.cloudfarming.client.crop.model.*;
import nl.cloudfarming.client.crop.type.CropsStorageDataObject;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.netbeans.core.spi.multiview.text.MultiViewEditorElement;
import org.openide.awt.UndoRedo;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 * 
 * Visual tab for crops. This panel enables users to create new crops and alter existing crops
 * @author Merijn Zengers
 */
@MultiViewElement.Registration(displayName = "#LBL_Crop_VISUAL",
iconBase = "nl/cloudfarming/client/stock/project/visual/box.png",
mimeType = "application/x-agrosense-stock",
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "CropVisual",
position = 2000)
@Messages({"LBL_Crop_VISUAL=Crop list"})
public class CropVisualElement extends MultiViewEditorElement {

    private transient MultiViewElementCallback callback;
    private CropsMainPanel cropPanel;
    private JToolBar toolbar = new JToolBar();
    private CropsStorage cropStorage;
    private CropsStorageDataObject cropsStorageDataObject;
    private CropService cropService;
    private Project project;

    public CropVisualElement(Lookup lkp) {
        super(lkp);
        FileObject stock = lkp.lookup(FileObject.class);
        project = null;
        try {
            project = ProjectManager.getDefault().findProject(stock.getParent());
        } catch (IOException | IllegalArgumentException ex) {
            Exceptions.printStackTrace(ex);
        }

        cropService = Lookup.getDefault().lookup(CropService.class);
        try {
            cropsStorageDataObject = (CropsStorageDataObject) cropService.getCropsStorageDataObject(project);
            cropStorage = cropService.getCropsStorage(project);
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
        cropPanel = new CropsMainPanel(cropStorage);
    }

    @Override
    public JComponent getVisualRepresentation() {
        return cropPanel;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), cropsStorageDataObject.getLookup());
    }

    @Override
    public void componentOpened() {
        cropPanel.comPonentOpened();
    }

    /**
     * When a component is closed empty rows must be removed
     */
    @Override
    public void componentClosed() {
        if (removeEmptyRows(cropsStorageDataObject.getCropsStorage())) {
            CropManager manager = Lookup.getDefault().lookup(CropManager.class);
            if (manager != null) {
                manager.save(cropsStorageDataObject.getCropsStorage(), cropsStorageDataObject.getPrimaryFile());
            }
        }
    }

    /**
     * Method which removes empty elements
     * Elements can be Crop Race or Genus
     * When a Crop name is empty and none of the names of the genera and Races are filled
     * the crop will be removed from cropsStorage
     * When the crop name is not filled filed but there is a genus with a name and a race without
     * the crop will be preserved but the race will be removed and vice versa
     * When a crop name is filled but there is a race and a genus both without a name
     * both the genus and the and the race are removed, the crop is preserved
     * @param cropsStorage the cropStorage to check
     * @return boolean if there is any change to cropsStorage
     */
    protected boolean removeEmptyRows(CropsStorage cropsStorage) {
        boolean changed = false;
        ArrayList<Crop> remove = new ArrayList<>();
        ArrayList<Genus> removedGenus = new ArrayList<>();
        ArrayList<Race> removedRace = new ArrayList<>();

        for (Crop crop : cropsStorage.getCrops()) {
            boolean genusRemove = true;
            for (Genus genus : crop.getGenera()) {
                if (genus.getName() != null && !genus.getName().equals("")) {
                    genusRemove = false;
                } else {
                    removedGenus.add(genus);
                    changed = true;
                }
            }
            crop.getGenera().removeAll(removedGenus);
            boolean raceRemove = true;
            for (Race race : crop.getRaces()) {
                if (race.getName() != null && !race.getName().equals("")) {
                    raceRemove = false;
                } else {
                    removedRace.add(race);
                    changed = true;
                }
            }
            crop.getRaces().removeAll(removedRace);


            if ((crop.getName() == null || crop.getName().equals(""))) {
                if (crop.getGenera().isEmpty() && crop.getRaces().isEmpty()) {
                    remove.add(crop);
                    changed = true;
                } else {
                    if (genusRemove && raceRemove) {
                        remove.add(crop);
                        changed = true;
                    }
                }
            }
        }
        for(Crop crop : remove){
            cropsStorage.removeCrop(crop);
        }
        return changed;
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }
    
    @Override
    public void updateName() {
        if (callback != null) {
            String name = project.getProjectDirectory().getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
        }
    } 

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return cropsStorageDataObject.getUndoRedo();
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

}
