/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project.visual;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.*;
import org.openide.util.NbBundle;

/**
 * 
 * @author Merijn Zengers & Gerben Feenstra
 */
@NbBundle.Messages({
    "CropPanelVisual.cropLabel=Name", 
    "CropPanelVisual.cropsHeader=Crops", 
    "CropPanelVisual.genusHeader=Class", 
    "CropPanelVisual.genusLabel=Name", 
    "CropPanelVisual.racesHeader=Races", 
    "CropPanelVisual.raceLabel=Name", 
    "CropPanelVisual.propertiesHeader=Properties"
})
public class CropsMainPanel extends JPanel implements PropertyChangeListener {

    private static final int LIST_WIDTH = 350;
    private static final int LIST_HEIGHT = 550;

    private final CropFactory cropFactory = new CropFactory();
    private final GenusFactory genusFactory = new GenusFactory();
    private final RaceFactory raceFactory = new RaceFactory();

    private ListController<Crop> cropsList;
    private ListController<Genus> genusList;
    private ListController<Race> racesList;
    
    private CropProperties cropPropertiesPanel;
    private JPanel cropDetailsPanel;
    
    private JScrollPane cropMainScrollPane;
    private JScrollPane cropDetailsScrollPane;
    
    private List<Crop> crops;
    private CropsStorage cropsStorage;

    // Panel headers
    private final String CROP_HEADER_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.cropsHeader");
    private final String GENUS_HEADER_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.genusHeader");
    private final String RACES_HEADER_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.racesHeader");
    private final String CROP_PROPERTIES_HEADER_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.propertiesHeader");

    // Column label
    private final String CROP_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.cropLabel");
    private final String GENUS_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.genusLabel");
    private final String RACES_LABEL = NbBundle.getMessage(CropsMainPanel.class, "CropPanelVisual.raceLabel");

    public CropsMainPanel(CropsStorage cropsStorage) {
        this.cropsStorage = cropsStorage;
        this.crops = cropsStorage.getCrops();
        init();
    }
    
    /**
     * This panel has 3 lists: Crops, Races and Classes
     */
    private void init() {
        this.setLayout(new MigLayout("wrap 1", "left, max", "top, max"));
        cropDetailsPanel = new JPanel(new MigLayout("wrap 1, insets 0 0 0 0"));
        cropDetailsPanel.setPreferredSize(new Dimension(LIST_WIDTH, LIST_HEIGHT));

        cropsList = new ListController<>(cropFactory, crops, CROP_HEADER_LABEL, CROP_LABEL);
        genusList = new ListController<>(genusFactory, crops.get(0).getGenera(), GENUS_HEADER_LABEL, GENUS_LABEL);
        racesList = new ListController<>(raceFactory, crops.get(0).getRaces(), RACES_HEADER_LABEL, RACES_LABEL);
        cropPropertiesPanel = new CropProperties(crops.get(0), CROP_PROPERTIES_HEADER_LABEL);
        
        cropsList.getComponent().setPreferredSize(new Dimension(LIST_WIDTH, LIST_HEIGHT));
        genusList.getComponent().setPreferredSize(new Dimension(LIST_WIDTH, new Double(LIST_HEIGHT * 0.4).intValue() ));
        racesList.getComponent().setPreferredSize(new Dimension(LIST_WIDTH, new Double(LIST_HEIGHT * 0.4).intValue() ));
        cropPropertiesPanel.setPreferredSize(new Dimension(LIST_WIDTH, new Double(LIST_HEIGHT * 0.2).intValue() ));
        
        cropsList.addPropertyChangeListener(ListController.PROP_SELECTION, this);

        cropsList.addPropertyChangeListener(ListController.PROP_ADD_ROW, this);
        genusList.addPropertyChangeListener(ListController.PROP_ADD_ROW, this);
        racesList.addPropertyChangeListener(ListController.PROP_ADD_ROW, this);

        cropsList.addPropertyChangeListener(ListController.PROP_REMOVE_ROW, this);
        genusList.addPropertyChangeListener(ListController.PROP_REMOVE_ROW, this);
        racesList.addPropertyChangeListener(ListController.PROP_REMOVE_ROW, this);

        cropsStorage.addPropertyChangeListener(CropsStorage.PROP_CROP, this);

        for (Crop crop : cropsStorage.getCrops()) {
            crop.addPropertyChangeListener(Crop.PROP_GENUS, this);
            crop.addPropertyChangeListener(Crop.PROP_RACE, this);
            crop.addPropertyChangeListener(Crop.PROP_NAME, this);
            for (Genus genus : crop.getGenera()) {
                genus.addPropertyChangeListener(Genus.PROP_NAME, this);
            }
            for (Race race : crop.getRaces()) {
                race.addPropertyChangeListener(Race.PROP_NAME, this);
            }
        }

        // Put components inside scrollpanes
        //this.add(cropsList.getComponent());
        cropMainScrollPane = new JScrollPane();
        cropMainScrollPane.setViewportView(cropsList.getComponent());

        // Put scrollpanes inside Splitpane
        cropDetailsPanel.add(racesList.getComponent());
        cropDetailsPanel.add(genusList.getComponent());
        cropDetailsPanel.add(cropPropertiesPanel);
        cropDetailsScrollPane = new JScrollPane();
        cropDetailsScrollPane.setViewportView(cropDetailsPanel);
        
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, cropsList.getComponent(), cropDetailsPanel);

        this.add(splitPane);        
        
    }
    
    public void comPonentOpened(){
        if (!crops.isEmpty()) {
                       
            cropsList.setSelectedBean(crops.get(0));
            
            // Races
            List<Race> races = crops.get(0).getRaces();
            racesList.setBeans(races);
            if(races != null && !races.isEmpty()){
                Race firstRace = races.get(0);
                if(firstRace != null){
                    racesList.setSelectedBean(firstRace);
                }
            }
            
            // Genera
            List<Genus> genera = crops.get(0).getGenera();
            genusList.setBeans(genera);
            if(genera != null && !genera.isEmpty()){
                Genus firstGenus = genera.get(0);
                if(firstGenus != null){
                    genusList.setSelectedBean(firstGenus);
                }
            }
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        switch (pce.getPropertyName()) {
            case ListController.PROP_SELECTION:
                if (pce.getNewValue() != null) {
                    genusList.setBeans(((Crop) pce.getNewValue()).getGenera());
                    genusList.setEnabled(true);
                    racesList.setBeans(((Crop) pce.getNewValue()).getRaces());
                    racesList.setEnabled(true);
                    cropPropertiesPanel.setCrop((Crop) pce.getNewValue());
                    cropPropertiesPanel.setEnabled(true);
                } else {
                    genusList.setBeans(Collections.EMPTY_LIST);
                    genusList.setEnabled(false);
                    racesList.setBeans(Collections.EMPTY_LIST);
                    racesList.setEnabled(false);
                    cropPropertiesPanel.setCrop(null);
                    cropPropertiesPanel.setEnabled(true);
                }
                break;
            case ListController.PROP_ADD_ROW:
                {
                    Crop selectedCrop = cropsList.getSelectedBean();
                    if (pce.getNewValue() instanceof Crop) {
                        cropsStorage.addCrop((Crop) pce.getNewValue());
                        ((Crop) pce.getNewValue()).addPropertyChangeListener(Crop.PROP_GENUS, this);
                        ((Crop) pce.getNewValue()).addPropertyChangeListener(Crop.PROP_RACE, this);
                    } else if (pce.getNewValue() instanceof Genus) {
                        if (selectedCrop != null) {
                            selectedCrop.addGenus((Genus) pce.getNewValue());
                            ((Genus) pce.getNewValue()).addPropertyChangeListener(Genus.PROP_NAME, this);
                        }
                    } else if (pce.getNewValue() instanceof Race) {
                        if (selectedCrop != null) {
                            selectedCrop.addRace((Race) pce.getNewValue());
                            ((Race) pce.getNewValue()).addPropertyChangeListener(Race.PROP_NAME, this);
                        }
                    }
                    break;
                }
            case ListController.PROP_REMOVE_ROW:
                {
                    Crop selectedCrop = cropsList.getSelectedBean();
                    if (pce.getOldValue() instanceof Crop) {
                        cropsStorage.removeCrop((Crop) pce.getOldValue());
                        ((Crop) pce.getOldValue()).removePropertyChangeListener(this);
                    } else if (pce.getOldValue() instanceof Genus) {
                        if (selectedCrop != null) {
                            selectedCrop.removeGenus((Genus) pce.getOldValue());
                            ((Genus) pce.getOldValue()).removePropertyChangeListener(this);
                        }
                    } else if (pce.getOldValue() instanceof Race) {
                        if (selectedCrop != null) {
                            selectedCrop.removeRace((Race) pce.getOldValue());
                            ((Race) pce.getOldValue()).removePropertyChangeListener(this);
                        }
                    }
                    break;
                }
            case CropsStorage.PROP_CROP:
                if (pce.getOldValue() instanceof List && pce.getNewValue() instanceof List) {
                    cropsList.setBeans((List) pce.getNewValue());
                } else if (pce.getNewValue() != null && pce.getNewValue() instanceof Crop) {
                    if (!((Crop) pce.getNewValue()).getSource().equals(Crop.USER_INPUT)) {
                        cropsList.addBean((Crop) pce.getNewValue());
                    }
                }
                break;
            case Crop.PROP_GENUS:
                if (pce.getOldValue() instanceof List && pce.getNewValue() instanceof List) {
                    genusList.setBeans((List) pce.getNewValue());
                }
                break;
            case Crop.PROP_RACE:
                if (pce.getOldValue() instanceof List && pce.getNewValue() instanceof List) {
                    racesList.setBeans((List) pce.getNewValue());
                }
                break;
        }
    }
}
