/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project.visual;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.CropBean;
import nl.cloudfarming.client.util.swing.button.ImageButton;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 * A row which displays two JTextfields
 * One for the name of the race provided to the constructor
 * And one for the name of the bean
 */
@Messages("CropPanelVisual.delete.button=delete")
public class Row<T extends CropBean> extends JPanel implements KeyListener, ActionListener, MouseListener, PropertyChangeListener {

    private T bean;
    private JTextField nameField;
    private static final String REMOVE_ICON = "delete.png";
    private static final String REMOVE_SELECTED_ICON = "delete_sel.png";
    private static final String REMOVE_DISABLED_ICON = "delete_dis.png";
    
    // Selection Icon resources
    private JLabel leftIconLabel = new JLabel();
    private static final String SELECTED_ICON_LEFT = "selected_icon_left.png";
    private static final String SELECTED_ICON_LEFT_FADED = "selected_icon_left_alpha.png";
    
    private ImageIcon leftIcon = createImageIcon(SELECTED_ICON_LEFT);
    private ImageIcon leftIconFaded = createImageIcon(SELECTED_ICON_LEFT_FADED);
    
    private final String DELETE_DESCRIPTION = NbBundle.getMessage(Row.class, "CropPanelVisual.delete.button");
    public static final int FIELD_WIDTH = 250;
    private List<ActionListener> listeners = new ArrayList<>();
    private boolean selected = false;
    private boolean hovered = false;
    public static final String PROP_SELECTED = "selected";
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.stock.project");    

    public Row(T bean) {
        this.bean = bean;
        init();
    }
    
    /**
     * returns if the row is selected.
     * @return true if selected else false
     */
    public boolean isSelected() {
        return selected;
    }

    /**
     * Set this row as selected
     * Will set the border if selected is true
     * @param selected if the row is selected
     */
    public void setSelected(boolean selected) {
        if (selected) {
            leftIconLabel.setIcon(leftIcon);
            leftIconLabel.setVisible(true);
        } else {
            leftIconLabel.setVisible(false);
        }
        this.selected = selected;
    }

    /**
     * return true if the row is hovered
     * @return boolean if the row is hovered
     */
    public boolean isHovered() {
        return hovered;
    }

    /**
     * Set if the row is hovered
     * @param hovered true if hovered else false
     */
    private void setHovered(boolean hovered) {
        if (hovered && !isSelected()) {
            leftIconLabel.setIcon(leftIconFaded);
            leftIconLabel.setVisible(true);
        } else if (!isSelected()){
            leftIconLabel.setVisible(false);
        }
        this.hovered = hovered;
    }

    /**
     * gets the Bean this row is representing
     * @return the bean contained in this row
     */
    public T getBean() {
        return this.bean;
    }

    /** Returns an ImageIcon, or null if the path was invalid. */
    protected ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = getClass().getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            LOGGER.log(Level.FINE, "Couldn''t find file: {0}", path);
            return null;
        }
    }

    private void init() {
        this.setLayout(new MigLayout("wrap 4, insets 0 10 10 10"));
        this.addMouseListener(this);
        
        // Selection Icon Left
        
        leftIconLabel.setIcon(leftIcon);
        leftIconLabel.setHorizontalAlignment(JLabel.RIGHT);
        leftIconLabel.setVisible(false);
        this.add(leftIconLabel);
        nameField = new JTextField(bean.getName());
        nameField.setPreferredSize(new Dimension(FIELD_WIDTH, 0));
        nameField.setFocusable(true);
        this.add(nameField);
        
        nameField.addKeyListener(this);
        nameField.addMouseListener(this);
        BufferedImage dellImage;
        BufferedImage dellSelImage;
        BufferedImage dellDisImage;
        try {
            dellImage = ImageIO.read(getClass().getResource(REMOVE_ICON));
            dellSelImage = ImageIO.read(getClass().getResource(REMOVE_SELECTED_ICON));
            dellDisImage = ImageIO.read(getClass().getResource(REMOVE_DISABLED_ICON));
            ImageButton removeButton = new ImageButton(dellImage, dellSelImage, dellDisImage,this.getBackground(), DELETE_DESCRIPTION);
            removeButton.addActionListener(this);
            removeButton.setFocusable(true);
            this.add(removeButton);
        } catch (IOException ex) {
            LOGGER.fine("Could not load image " + REMOVE_ICON);
        }
        
        this.bean.addPropertyChangeListener(CropBean.PROP_NAME, this);
    }


    @Override
    public void keyTyped(KeyEvent ke) {
    }

    @Override
    public void keyPressed(KeyEvent ke) {
    }

    @Override
    public void keyReleased(KeyEvent ke) {
        if (ke.getSource().equals(nameField)) {
            bean.setName(nameField.getText());
        }
    }

    public void addActionListener(ActionListener actionListener) {
        this.listeners.add(actionListener);
    }

    public void removeActionListeners(ActionListener actionListener) {
        this.listeners.remove(actionListener);
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
        for (ActionListener listener : listeners) {
            ae.setSource(this);
            listener.actionPerformed(ae);
        }
    }

    @Override
    public void mouseClicked(MouseEvent me) {
        boolean old = this.selected;
        this.firePropertyChange(PROP_SELECTED, old, true);
    }

    @Override
    public void mousePressed(MouseEvent me) {
    }

    @Override
    public void mouseReleased(MouseEvent me) {
        boolean old = this.selected;
        this.firePropertyChange(PROP_SELECTED, old, true);
    }

    @Override
    public void mouseEntered(MouseEvent me) {
        setHovered(true);
    }

    @Override
    public void mouseExited(MouseEvent me) {
        setHovered(false);
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        nameField.setText(this.bean.getName());
    }
}
